<link rel="stylesheet" href="https://www.google.com/cse/style/look/default.css" type="text/css" />
<style type="text/css">
    a.navbar {
    color: {{ theme_linkcolor }};
    letter-spacing: .05em;
    font-weight: bold;
        }
</style>

<a class="navbar" href="{{pathto('index')}}">Home</a> ·
<a class="navbar" href="{{pathto('quickstart')}}">Quickstart</a> ·
<a class="navbar" href="{{pathto('documentation')}}">Documentation</a> ·
<a class="navbar" href="{{pathto('about')}}">About</a> ·
<a class="navbar" href="http://nipy.org">Nipy</a>

