# AUTO-GENERATED by tools/checkspecs.py - DO NOT EDIT
from __future__ import unicode_literals
from ..diffusion import DiffusionWeightedVolumeMasking


def test_DiffusionWeightedVolumeMasking_inputs():
    input_map = dict(
        args=dict(argstr='%s', ),
        environ=dict(
            nohash=True,
            usedefault=True,
        ),
        inputVolume=dict(
            argstr='%s',
            extensions=None,
            position=-4,
        ),
        otsuomegathreshold=dict(argstr='--otsuomegathreshold %f', ),
        outputBaseline=dict(
            argstr='%s',
            hash_files=False,
            position=-2,
        ),
        removeislands=dict(argstr='--removeislands ', ),
        thresholdMask=dict(
            argstr='%s',
            hash_files=False,
            position=-1,
        ),
    )
    inputs = DiffusionWeightedVolumeMasking.input_spec()

    for key, metadata in list(input_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(inputs.traits()[key], metakey) == value
def test_DiffusionWeightedVolumeMasking_outputs():
    output_map = dict(
        outputBaseline=dict(
            extensions=None,
            position=-2,
        ),
        thresholdMask=dict(
            extensions=None,
            position=-1,
        ),
    )
    outputs = DiffusionWeightedVolumeMasking.output_spec()

    for key, metadata in list(output_map.items()):
        for metakey, value in list(metadata.items()):
            assert getattr(outputs.traits()[key], metakey) == value
