@import url("./sphinxdoc.css");

body {
    background-color: #E5E2E2;
    min-width: 70em;
}

div.sphinxsidebar h4, div.sphinxsidebar h3 {
    background-color: #2F83C8;
}

h1 {
    color: #2F83C8;
}

div.footer {
    background-color: #CEC890;
}


div.body {
    background-color: white;
}

div.highlight-python pre {
white-space: pre-wrap; /* css-3 */
white-space: -moz-pre-wrap !important; /* Mozilla, since 1999 */
white-space: -pre-wrap; /* Opera 4-6 */
white-space: -o-pre-wrap; /* Opera 7 */
word-wrap: break-word; /* Internet Explorer 5.5+ */
}

p.graphviz img {
    max-width: 100%;
}

th.field-name {
    white-space: nowrap;
}

div.doc2.container tbody {
    vertical-align: top;
}

div.doc2.container li {
    list-style-type: none;
}

div.doc2.container p.first{
    font-weight: bold;
    font-size: 14px;
}

div.doc2.container ul{
    margin: 10px;
}

div.doc2 .reference.internal{
    font-size: 14px;
}
